from django.contrib import admin
from models import PushNotification, Device


class PushNotificationAdmin(admin.ModelAdmin):
    list_display = ('title', 'date_created', 'active', 'sent')
    list_filter = ('active', 'sent')
    search_fields = ('title', )


class DeviceAdmin(admin.ModelAdmin):
    list_display = ('key', )
    list_filter = ('user', )


admin.site.register(PushNotification, PushNotificationAdmin)
admin.site.register(Device, DeviceAdmin)


from django.db import models
from django.contrib.auth import get_user_model
from django.utils.translation import ugettext_lazy as _


class PushNotification(models.Model):
    PUSH_INACTIVE = 0
    PUSH_ACTIVE = 1

    PUSH_NOT_SENT = 0
    PUSH_SENT = 1

    PUSH_CHOICES = (
        (PUSH_ACTIVE, _('Active')),
        (PUSH_INACTIVE, _('Inactive'))
    )

    PUSH_SENT_CHOICES = (
        (PUSH_NOT_SENT, _('Not Sent')),
        (PUSH_SENT, _('Sent'))
    )

    title = models.CharField(max_length=50)
    body = models.TextField()

    active = models.SmallIntegerField(choices=PUSH_CHOICES, default=PUSH_ACTIVE)
    sent = models.SmallIntegerField(choices=PUSH_SENT_CHOICES, default=PUSH_NOT_SENT)

    date_created = models.DateTimeField(auto_now_add=True)

    def __unicode__(self):
        return self.title


class Device(models.Model):
    DEVICE_TYPE_ANDROID = 1
    DEVICE_TYPE_IOS = 2
    DEVICE_TYPE_CHOICES = (
        (DEVICE_TYPE_ANDROID, _('Android')),
        (DEVICE_TYPE_IOS, _('iOS'))
    )

    key = models.TextField()
    type = models.SmallIntegerField(choices=DEVICE_TYPE_CHOICES)
    user = models.ForeignKey(get_user_model(), blank=True, null=True)

    class Meta:
        unique_together = ('key', 'type')


from models import PushNotification
from tasks import create_push_notification_groups


def send_push_notification(title, body):
    push_notification = PushNotification(title=title,
                                         body=body,
                                         active=PushNotification.PUSH_ACTIVE,
                                         sent=PushNotification.PUSH_NOT_SENT)
    push_notification.save()

    create_push_notification_groups.delay(notification_id=push_notification.id)


from gcm import GCM
from gcm.gcm import GCMNotRegisteredException

from django.conf import settings

from models import Device
from exceptions import PushException, PushGCMApiKeyException

dispatchers_cache = {}


class Dispatcher(object):
    PUSH_RESULT_SENT = 1
    PUSH_RESULT_NOT_REGISTERED = 2
    PUSH_RESULT_EXCEPTION = 3

    def send(self, device_key, data):
        raise NotImplemented()


class APNSDispatcher(Dispatcher):
    def send(self, device_key, data):
        pass


class GCMDispatcher(Dispatcher):
    def send(self, device_key, data):
        gcm_api_key = getattr(settings, 'PUSHY_GCM_API_KEY', None)
        if not gcm_api_key:
            raise PushGCMApiKeyException()

        gcm = GCM(gcm_api_key)

        # Plaintext request
        try:
            canonical_id = gcm.plaintext_request(registration_id=device_key, data=data)
            if canonical_id:
                return self.PUSH_RESULT_SENT, canonical_id
            else:
                return self.PUSH_RESULT_SENT, 0
        except GCMNotRegisteredException:
            return self.PUSH_RESULT_NOT_REGISTERED, 0
        except IOError:
            return self.PUSH_RESULT_EXCEPTION, 0
        except PushException:
            return self.PUSH_RESULT_EXCEPTION, 0


def get_dispatcher(device_type):
    if device_type in dispatchers_cache and dispatchers_cache[device_type]:
        return dispatchers_cache[device_type]

    if device_type == Device.DEVICE_TYPE_ANDROID:
        dispatchers_cache[device_type] = GCMDispatcher()
    elif device_type == Device.DEVICE_TYPE_IOS:
        dispatchers_cache[device_type] = APNSDispatcher()

    return dispatchers_cache[device_type]




from setuptools import setup

setup(
    name='Django-Pushy',
    version='0.1.0',
    author='Rakan Alhneiti',
    author_email='rakan.alhneiti@gmail.com',

    # Packages
    packages=['pushy'],
    include_package_data=True,

    # Details
    url='https://github.com/rakanalh/django-pushy',

    license='LICENSE.txt',
    description='Handle push notifications at scale.',
    long_description=open('README.rst').read(),

    # Dependent packages (distributions)
    install_requires=[
        'python-gcm',
        'django',
        'django-celery'
    ],
)